@@ -4,25 +4,15 @@ import warnings
 from typing import Dict, List, Optional, Union
 
 from . import aws_connection, connection, mqtt_connection
-from .status import (
-    JciHitachiCommand,
-    JciHitachiCommandAC,
-    JciHitachiCommandDH,
-    JciHitachiStatusInterpreter,
-)
-from .model import (
-    JciHitachiStatusSupport,
-    JciHitachiACSupport,
-    JciHitachiDHSupport,
-    JciHitachiStatus,
-    JciHitachiAC,
-    JciHitachiDH,
-    JciHitachiAWSStatusSupport,
-    JciHitachiAWSStatus,
-)
+from .model import (JciHitachiAC, JciHitachiACSupport, JciHitachiAWSStatus,
+                    JciHitachiAWSStatusSupport, JciHitachiDH,
+                    JciHitachiDHSupport, JciHitachiStatus,
+                    JciHitachiStatusSupport)
+from .status import (JciHitachiCommand, JciHitachiCommandAC,
+                     JciHitachiCommandDH, JciHitachiStatusInterpreter)
 
 
-class Peripheral:
+class Peripheral: # pragma: no cover
     """Peripheral (Device) Information.
 
     Parameters
@@ -273,7 +263,7 @@ class Peripheral:
         )
 
 
-class JciHitachiAPI:
+class JciHitachiAPI: # pragma: no cover
     """Jci-Hitachi API.
 
     Parameters
@@ -615,7 +605,7 @@ class AWSThing:
     Parameters
     ----------
     thing_json : dict
-        Thing json of specific device.
+        Thing json of a specific device.
     """
 
     supported_device_type = {
@@ -641,6 +631,7 @@ class AWSThing:
             f"available: {self.available}\n"
             f"status_code: {self.status_code}\n"
             f"support_code: {self.support_code}\n"
+            f"shadow: {self.shadow}\n"
             f"gateway_mac_address: {self.gateway_mac_address}"
         )
         return ret
@@ -864,11 +855,11 @@ class JciHitachiAWSAPI:
         self.print_response : bool = print_response
 
         self._mqtt : Optional[aws_connection.JciHitachiAWSMqttConnection] = None
+        self._shadow_names : Union[str, list] = ["info"]
         self._device_id : int = random.randint(1000, 6999)
         self._things : Dict[str, AWSThing] = {}
         self._aws_tokens : Optional[aws_connection.AWSTokens] = None
         self._aws_identity : Optional[aws_connection.AWSIdentity] = None
-        self._aws_credentials : Optional[aws_connection.AWSCredentials] = None
         self._host_identity_id : Optional[str] = None
         self._task_id : int = 0
 
@@ -898,15 +889,14 @@ class JciHitachiAWSAPI:
         return self._task_id
 
     def _check_before_publish(self) -> None:
-        # Reauthenticate 5 mins before AWSTokens or AWSCredentials expiration.
+        # Reauthenticate 5 mins before AWSTokens expiration.
         current_time = time.time()
-        if self._aws_tokens.expiration - current_time <= 300 or self._aws_credentials.expiration - current_time <= 300:
+        if self._aws_tokens.expiration - current_time <= 300:
             self.reauth()
 
         if self._mqtt.mqtt_events.mqtt_error_event.is_set():
-            if self._mqtt.mqtt_events.mqtt_error == "wssHandShakeError":
-                self._mqtt.mqtt_events.mqtt_error_event.clear()
-                self.reauth()
+            self._mqtt.mqtt_events.mqtt_error_event.clear()
+            self.reauth()
         
         # Clear events
         self._mqtt.mqtt_events.device_control_event.clear()
@@ -934,14 +924,6 @@ class JciHitachiAWSAPI:
         self._aws_tokens = conn.aws_tokens
         conn_status, self._aws_identity = conn.get_data()
 
-        conn = aws_connection.GetCredentials(
-            email=self.email,
-            password=self.password,
-            aws_tokens=self._aws_tokens,
-            print_response=self.print_response,
-        )
-        conn_status, self._aws_credentials = conn.get_data(self._aws_identity)
-
         conn = aws_connection.ListSubUser(self._aws_tokens, print_response=self.print_response)
         conn_status, conn_json = conn.get_data()
 
@@ -966,14 +948,28 @@ class JciHitachiAWSAPI:
             thing_names = [value.thing_name for value in self._things.values()]
 
             # mqtt
-            self._mqtt = aws_connection.JciHitachiAWSMqttConnection(self._aws_credentials, print_response=self.print_response)
+            def get_credential_callable():
+                conn = aws_connection.GetCredentials(
+                    email=self.email,
+                    password=self.password,
+                    aws_tokens=self._aws_tokens,
+                    print_response=self.print_response,
+                )
+                conn_status, aws_credentials = conn.get_data(self._aws_identity)
+                if conn_status != "OK":
+                    aws_connection._LOGGER.error(f"An error occurred when acquiring a new AwsCredentials: {conn_status}")
+                return aws_credentials
+
+            self._mqtt = aws_connection.JciHitachiAWSMqttConnection(get_credential_callable, print_response=self.print_response)
             self._mqtt.configure()
-            self._mqtt.connect(self._host_identity_id, thing_names)
+            
+            if not self._mqtt.connect(self._host_identity_id, self._shadow_names, thing_names):
+                raise RuntimeError(f"An error occurred when connecting to MQTT endpoint.")
 
             # status
             self.refresh_status(refresh_support_code=True, refresh_shadow=True)
         else:
-            raise RuntimeError(f"An error occurred when retrieving device info: {conn_status}")
+            raise RuntimeError(f"An error occurred when retrieving devices info: {conn_status}")
     
     def logout(self) -> None:
         """Logout API.
@@ -982,6 +978,8 @@ class JciHitachiAWSAPI:
         self._mqtt.disconnect()
 
     def reauth(self) -> None:
+        """Reauthenticate with AWS Cognito Service."""
+
         conn = aws_connection.JciHitachiAWSCognitoConnection(
             email=self.email,
             password=self.password,
@@ -989,21 +987,8 @@ class JciHitachiAWSAPI:
             print_response=self.print_response,
         )
         conn_status, self._aws_tokens = conn.login(use_refresh_token=False)
-
-        conn = aws_connection.GetCredentials(
-            email=self.email,
-            password=self.password,
-            aws_tokens=self._aws_tokens,
-            print_response=self.print_response,
-        )
-        conn_status, self._aws_credentials = conn.get_data(self._aws_identity)
-
-        thing_names = [value.thing_name for value in self._things.values()]
-
-        self._mqtt.aws_credentials = self._aws_credentials
-        self._mqtt.disconnect()
-        self._mqtt.configure()
-        self._mqtt.connect(self._host_identity_id, thing_names)
+        if conn_status != "OK":
+            raise RuntimeError(f"An error occurred when reauthenticating with AWS Cognito Service: {conn_status}")
 
     def change_password(self, new_password: str) -> None:
         """Change password. 
@@ -1130,7 +1115,7 @@ class JciHitachiAWSAPI:
             
             self._mqtt.mqtt_events.device_status_event.clear()
 
-    def get_status(self, device_name: Optional[str] = None, legacy=False) -> Dict[str, JciHitachiAWSStatus]:
+    def get_status(self, device_name: Optional[str] = None, legacy: bool = False) -> Dict[str, JciHitachiAWSStatus]:
         """Get device status after refreshing status.
 
         Parameters
@@ -1140,36 +1125,37 @@ class JciHitachiAWSAPI:
             If None is given, all devices' status will be returned,
             by default None.
         legacy : bool, optional
-            Whether or not to return legacy (old) status class, by default False.
+            Whether or not to return status with legacy status name, by default False.
 
         Returns
         -------
-        dict of JciHitachiAWSStatus or JciHitachiStatus.
-            If legacy is True, return a dict of JciHitachiStatus; otherwise, return a dict of JciHitachiAWSStatus instances.
+        dict of JciHitachiAWSStatus.
+            A dict of JciHitachiAWSStatus instances.
         """
         
         statuses = {}
         for name, thing in self._things.items():
-            if (device_name and name != device_name) or \
-                    thing.type == "unknown":
+            if (device_name and name != device_name) or thing.type == "unknown":
                 continue
             if legacy:
-                statuses[name] = thing.status_code.legacy_status_class
+                statuses[name] = thing.status_code.legacy_status
             else:
                 statuses[name] = thing.status_code
         return statuses
 
-    def set_status(self, status_name: str, status_value: int, device_name: str) -> bool:
-        """Set status to a thing.
+    def set_status(self, status_name: str, device_name: str, status_value: int = None, status_str_value: str = None) -> bool:
+        """Set status to a thing. Either status_value or status_str_value must be specified.
 
         Parameters
         ----------
         status_name : str
             Status name.
-        status_value : int
-            Status value.
         device_name : str
             Device name.
+        status_value : int, optional
+            Status value, by default None.
+        status_str_value : str, optional
+            Status string value, by default None.
 
         Returns
         -------
@@ -1185,8 +1171,17 @@ class JciHitachiAWSAPI:
         self._check_before_publish()
 
         thing = self._things[device_name]
-        if status_name not in JciHitachiAWSStatus.compatibility_mapping[thing.type]:
-            status_name = JciHitachiAWSStatus.convert_old_to_new(thing.type, status_name)
+
+
+        is_valid, status_name, status_value = JciHitachiAWSStatus.str2id(
+            device_type = thing.type, 
+            status_name = status_name,
+            status_value = status_value,
+            status_str_value = status_str_value,
+        )
+
+        if not is_valid:
+            return False
 
         shadow_publish_mapping = {
             "CleanFilterNotification": "filter",
@@ -1240,7 +1235,7 @@ class JciHitachiAWSAPI:
             if self._mqtt.mqtt_events.device_control_event.wait(timeout=10.0):
                 device_control = self._mqtt.mqtt_events.device_control.get(thing.thing_name)
                 if device_control.get(status_name) == status_value:
-                    thing.status_code._status[status_name] = status_value
+                    thing.status_code.set_new_status(status_name, status_value)
                     self._mqtt.mqtt_events.device_control_event.clear()
                     return True
             self._delay()
